# Reseda version 1.0.0
# MusetSet.py
# by Antoine Rauzy
# Copyrights (c) 2018-2021 AltaRica Association
#
# This file is part of Reseda.
# Reseda is free software: you can redistribute it and/or modify
# it under the terms of the GNU General Public License as published by
# the Free Software Foundation, either version 3 of the License, or
# (at your option) any later version.
# Reseda is distributed in the hope that it will be useful,
# but WITHOUT ANY WARRANTY; without even the implied warranty of
# MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
# GNU General Public License for more details.
# You should have received a copy of the GNU General Public License
# along with Reseda.  If not, see <http://www.gnu.org/licenses/>.

# Table of Contents
# -----------------
# 1) Imported modules
# 2) Creators
# 3) Accessors

# 1) Imported modules
# -------------------

import time

from S2MLCore import *
from ResedaModels import *
from MusetElement import *

# 2) Creators
# -----------

class MusetSet(MusetElement):

  def __init__(self):
    MusetElement.__init__(self, MusetElement.SET, None)
    self._elements = []

  @staticmethod
  def NewEmptySet():
    self = MusetSet()
    return self

  @staticmethod
  def NewSingleton(element):
    self = MusetSet()
    self.AppendElement(element)
    return self

  @staticmethod
  def Copy(other):
    self = MusetSet()
    for element in other._elements:
      self._elements.append(element)
    return self

# 3) Accessors
# ------------

  def GetElements(self):
    return self._elements

  def IsEqual(self, other):
    if self.GetSize()!=other.GetSize():
      return False
    return self.DoesIncludeSet(other)

  def GetSize(self):
    return len(self._elements)

  def DoesContainBlock(self, block):
    for element in self._elements:
      if element.GetBlock()==block:
        return True
    return False

  def DoesContainNode(self, node):
    for element in self._elements:
      if element.GetNode()==node:
        return True
    return False

  def DoesContainEdge(self, edge):
    for element in self._elements:
      if element.GetEdge()==edge:
        return True
    return False

  def DoesContainElement(self, candidateElement):
    for element in self._elements:
      if element.IsEqual(candidateElement):
        return True
    return False

  def DoesIncludeSet(self, other):
    if self.GetSize()<other.GetSize():
      return False
    for element in other._elements:
      if not self.DoesContainElement(element):
        return False
    return True

  def AppendElement(self, element):
     self._elements.append(element)

  def InsertElement(self, element):
    if self.DoesContainElement(element):
      return
    self._elements.append(element)

  def InsertSet(self, other):
    for element in other._elements:
      self.InsertElement(element)

  def RemoveElement(self, element):
    if not self.DoesContainElement(element):
      return
    self._elements.remove(element)

  def RemoveSet(self, other):
    for element in other._elements:
      self.RemoveElement(element)

  def Flush(self):
    self._elements.clear()
